module budget_fine
  !
  ! this budget routine defines the budgets for the finest zoom region only.
  ! (problem: two equivalent fine regions...? to be resolved)
  ! This will enable an analysis on regional basis (parts of this zoom area).
  ! Remember that there exists a 'core' zoom region and an inteface region.
  ! The budgets of the interface region are 'mixed' due to the algorithm.
  ! Emissions, Chemistry, etc.. are performed on the parent, which are then
  ! transported to the zoom region by put_xedges/put_yedges.
  ! For this region, the interface will be a separate budget-class.
  ! In the core region, the choice is free.
  ! nbud_v : vertical zones for budget calculations zone are defined &
  !          in ini_zone
  ! 
  use dims
  use chem_param, only : names, ra, nmark, marknam, njnum
  use chem_param, only : jnam, nreac, ratnam, nhet, rhetnam, ntracet, ntrace

  implicit none

  ! --- interface ---

  private

  ! public routines
  public :: ini_zone, diagbud, budget_transport

  ! public variables
  public :: nzon, nzon_v
  public :: budadvx, budadvy, budemi, budrj, budrr, budrw, budmark, budstrat
  public :: depwet_cp, depwet_lsp, depwet_nat, depdry


  ! --- module variables ---

  integer,parameter                     :: nbud=5
  integer,parameter                     :: nbud_v=3 

  ! wet and dry deposition accumulators
  real,dimension(im(nregions),jm(nregions),nbud_v,ntrace)  :: depwet_lsp
  real,dimension(im(nregions),jm(nregions),nbud_v,ntrace)  :: depwet_cp
  real,dimension(im(nregions),jm(nregions),nbud_v,ntrace)  :: depwet_nat
  real,dimension(im(nregions),jm(nregions),ntrace)         :: depdry
  real,dimension(im(nregions),jm(nregions),nbud_v,ntrace)  :: depwet_lsp_all
  real,dimension(im(nregions),jm(nregions),nbud_v,ntrace)  :: depwet_cp_all
  real,dimension(im(nregions),jm(nregions),nbud_v,ntrace)  :: depwet_nat_all
  real,dimension(im(nregions),jm(nregions),ntrace)         :: depdry_all

  ! to store acc. concentrations
  real,dimension(nbud,nbud_v,ntracet)                      :: rmold
  real,dimension(nbud,nbud_v,ntracet)                      :: rmnew 

  !    budrr  : accumulator for chemical reaction fluxes
  !    budrj  : accumulator for photolysis reactions
  !    buddry : accumulator for dry deposition losses
  !    budwet : accumulator for wet deposition losses
  !    budemi : accumulator for emissions entering calculations
  !    budstrat : accumulator for stratospheric losses and gains
  !    budconc  : average concentrations
  !    budchng  : change of concentration
  !    budini   : initial concentration at begin of run
  !    budadvx : advection in and out of region per tracer in x
  !    budadvy : advection in and out of region per tracer in y
  !    budadvz : advection in and out of region per tracer in z
  !    budconv : convectionin and out of region per tracer
  !
  real, dimension(nbud,nbud_v,nreac)  :: budrr
  real, dimension(nbud,nbud_v,nreac)  :: budrr_all
  real, dimension(nbud,nbud_v,njnum)  :: budrj
  real, dimension(nbud,nbud_v,njnum)  :: budrj_all
  real, dimension(nbud,nbud_v,ntrace) :: buddry
  real, dimension(nbud,nbud_v,ntrace) :: budwet_lsp
  real, dimension(nbud,nbud_v,ntrace) :: budwet_cp
  real, dimension(nbud,nbud_v,ntrace) :: budwet_nat
  real, dimension(nbud,nbud_v,ntrace) :: budemi
  real, dimension(nbud,nbud_v,ntrace) :: budstrat
  real, dimension(nbud,nbud_v,ntrace) :: budconc
  real, dimension(nbud,nbud_v,ntrace) :: budini
  real, dimension(nbud,nbud_v,ntrace) :: buddry_all
  real, dimension(nbud,nbud_v,ntrace) :: budwet_lsp_all
  real, dimension(nbud,nbud_v,ntrace) :: budwet_cp_all
  real, dimension(nbud,nbud_v,ntrace) :: budwet_nat_all
  real, dimension(nbud,nbud_v,ntrace) :: budemi_all
  real, dimension(nbud,nbud_v,ntrace) :: budstrat_all
  real, dimension(nbud,nbud_v,ntrace) :: budconc_all
  real, dimension(nbud,nbud_v,ntrace) :: budini_all
  real, dimension(nbud,nbud_v,ntrace) :: budchng = 0.0
  real, dimension(nbud,nbud_v,ntrace) :: budchng_all = 0.0
  real, dimension(nbud,nbud_v,nmark)  :: budmark
  real, dimension(nbud,nbud_v,nmark)  :: budmark_all
  real, dimension(nbud,nbud_v,nhet) :: budrw
  real, dimension(nbud,nbud_v,nhet) :: budrw_all
  real, dimension(nbud,nbud_v,ntracet):: budadvx,budadvy,budadvz
  real, dimension(nbud,nbud_v,ntracet):: budadvx_all,budadvy_all,budadvz_all
  real, dimension(nbud,nbud_v,ntracet):: budconv
  real, dimension(nbud,nbud_v,ntracet):: budconv_all

  character(len=20), dimension(nbud)           :: zone_name
  character(len=20), dimension(nbud_v)         :: zone_namv
  integer,dimension(im(nregions),jm(nregions)) :: nzon
  integer,dimension(lm(nregions))              :: nzon_v
  !WP! for offset on level domain
  integer                                      :: lglob 
  

contains



  subroutine ini_zone
    !
    ! user defined structure of budget zones
    !
    use io_hdf,    only : io_write, DFACC_CREATE, DFNT_INT32
    use toolbox,   only : lvlpress
    use global_data, only : outdir
#ifdef MPI
    use mpi_const, only : root,myid
#endif

    implicit none

    ! local
    integer :: i,j,n, nix,niy,imr,jmr,region
    logical :: x_interface = .true., y_interface = .true.
    integer :: io,sfstart,sfend,istat,sfsnatt
    integer :: ilvl1, ilvl2
    real    :: psurf, plev

    region = nregions

    imr = im(region)   !take the finest resolution for the budget
    jmr = jm(region) 

    nix = xref(region)/xref(parent(region))
    niy = yref(region)/yref(parent(region))    !number of cells in interface

    ! check where interfaces are
    if ( xcyc(region) == 1 ) x_interface = .false.
    if ( niy == 1 ) y_interface = .false.
    !
    zone_namv(1)='BL'
    zone_namv(2)='FT'
    zone_namv(3)='STRAT'
    !
    zone_name(1)='lower_l'
    zone_name(2)='lower_r'
    zone_name(3)='upper_l'
    zone_name(4)='upprr_r'
    zone_name(5)='interface'

    psurf = 98400.0
    plev  = 85000.0
    ilvl1 = lvlpress(nregions,plev, psurf)
    plev  = 10000.0
    ilvl2 = lvlpress(nregions,plev, psurf)
    nzon_v(1:ilvl1) = 1 !boundary layer 
    nzon_v(ilvl1+1:ilvl2)= 2 !free troposphere/lower stratosphere upto 100 hPa
    nzon_v(ilvl2+1:lm(1))=3 !stratosphere

    nzon(1:imr/2,1:jmr/2) = 1
    nzon(imr/2+1:imr,1:jmr/2) = 2
    nzon(1:imr/2,jmr/2+1:jmr) = 3
    nzon(imr/2+1:imr,jmr/2+1:jmr) = 4

    ! overwrite these squares with the interface....!
    if ( x_interface ) then
       nzon(1:nix,:) = 5
       nzon(imr-nix+1:imr,:) = 5
    end if
    if ( y_interface ) then
       if ( touch_sp(region) /= 1 ) nzon(:,1:niy) = 5
       if ( touch_np(region) /= 1 ) nzon(:,jmr-niy+1:jmr) = 5
    end if

#ifdef MPI
    if ( myid == root ) then
#endif
       io=SFSTART(trim(outdir)//'/regions_'//region_name(region)//'.hdf', DFACC_CREATE)
       istat = sfsnatt(io,'im',    DFNT_INT32, 1, imr)
       istat = sfsnatt(io,'jm',    DFNT_INT32, 1, jmr)
       istat = sfsnatt(io,'lm',    DFNT_INT32, 1, lm(region))
       call io_write(io,imr,'im',jmr,'jm', nzon,'region_fine')
       print*,'ini_zone: closing output file',sfend(io)
#ifdef MPI
    end if
#endif

  end subroutine ini_zone
  


  subroutine diagbud(region,k)
    !
    !
    !       interface
    !       ---------
    !       call call_diagbud(region,k)
    !       if k=1   reset budget variables budrr,budrj,buddry,
    !                budwet,budemi,budstrat,budconc,budchng, initialise budini
    !       if k=2   output of budget variables
    !       if k=3   accumulate deposition budget
    !
    !       method
    !       ------
    !       output of time accumulated budget files for postprocessing
    !
    !       externals
    !       ---------
    !       none
    !
    !       reference
    !       ---------
    !       none
    !-----------------------------------------------------------------------

    use global_data, only : outdir
    use global_data, only : mass_dat
    use io_hdf,      only : io_write
    use io_hdf,      only : DFACC_CREATE, DFNT_INT64, DFNT_INT32, DFNT_CHAR
    use toolbox,     only : escape_tm
#ifdef MPI
    use mpi_const
#endif

    implicit none

    ! in/out
    integer,intent(in) :: region,k

    ! local
    save
    real,dimension(:,:,:,:),pointer    :: rm
    integer :: nzone,nzone_v,j1
    integer :: io,istat,sfstart,sfend,sfsnatt,sfscatt 
    integer :: i,j,l,n,nt,iy

    integer,parameter  :: nproces=15     !number of budget processes

    character(len=10), dimension(nproces),parameter :: proces_name = (/&
         'ratnam    ','jnam      ','rhetnam   ','marknam   ','names     ',&
         'names     ','names     ','names     ','names     ','names     ',&
         'names     ','names     ','names     ','names     ','names     ' /)

    integer,dimension(nproces),parameter :: proces_nr = (/ &
         nreac,    njnum,  nhet,   nmark,  ntrace, &
         ntrace,  ntrace,  ntrace,  ntrace,  ntrace, &
         ntrace, ntracet, ntracet, ntracet, ntracet    /)

    integer :: communicator,offsetn,offsetl,lglob,nglob,root_id,lmr

    ! start

    ! if not last child return...

    !WP diagbud can be called during chemistry, whcih is 
    !   parallel over levels. However, a version parallel over 
    !WP tracers is also implemented for future changes.


#ifdef MPI
    which_par=previous_par(region)

    if ( which_par == 'tracer' .and. ntracetloc == 0 ) return 
    if ( which_par == 'levels' .and. lmloc == 0 ) return  !WP! 

    if (which_par=='tracer') then

       rm => mass_dat(region)%rm_t
       lmr = lm(region)
       nt=ntracetloc
       communicator=com_trac  !WP! assign com_trac as communicator
       root_id=root_t
       offsetn=sum(ntracet_ar(0:myid-1) )  !offset for global value of n
       offsetl=0  ! no offset for levels
    else if ( which_par == 'levels' ) then

       rm => mass_dat(region)%rm_k
       lmr = lmloc
       nt=ntracet
       communicator=com_lev  !WP! assign com_lev as communicator
       root_id=root_k
       offsetl=sum(lmar(0:myid-1) )  ! offset for global value of l
       offsetn=0    ! no offset for tracers

    end if
#else
    rm => mass_dat(region)%rm_t
    lmr = lm(region)
    nt=ntracet
    offsetn=0  !
    offsetl=0  ! no offset for levels
#endif

    ! this budget routine only in the finest region!
    if ( region /= nregions ) return

    select case(k)

    case(1)

#ifdef MPI
       if ( which_par /= 'tracer' ) then
          call escape_tm('diagbud: initializing budgets should be done on tracer domain')
       end if
#endif

       write(kmain,*) 'diagbud: reset budget variables'
       budrr=0.
       budrr_all=0.
       budrj=0.
       budrj_all=0.
       buddry=0.
       buddry_all=0.
       budwet_lsp=0.
       budwet_lsp_all=0.
       budwet_cp=0.
       budwet_cp_all=0.
       budwet_nat=0.
       budwet_nat_all=0.
       budemi=0.
       budemi_all=0.
       budstrat=0.
       budstrat_all=0.
       budconc=0.
       budconc_all=0.
       budini=0.
       budini_all=0.
       budmark=0.
       budmark_all=0.
       budrw=0.
       budrw_all=0.
       budadvx=0.0
       budadvx_all=0.0
       budadvy=0.0
       budadvy_all=0.0
       budadvz=0.0
       budadvz_all=0.0
       budconv=0.0
       budconv_all=0.0
       !
       depwet_lsp=0.
       depwet_lsp_all=0.
       depwet_cp=0.
       depwet_cp_all=0.
       depwet_nat=0.
       depwet_nat_all=0.
       depdry=0.  
       depdry_all=0.  

       !
       ! initialise budchange
       !

       do n=1,nt
          nglob=n+offsetn
          do l=1,lmr
             nzone_v=nzon_v(l)
             do j=1,jm(region)
                do i=1,im(region)
                   nzone=nzon(i,j)
                   budchng(nzone,nzone_v,nglob) = &
                        budchng(nzone,nzone_v,nglob) + &
                        rm(i,j,l,n)/ra(nglob)*1000.
                end do !i
             end do!j
          end do!l
       end do !n  
       nullify(rm)       

    case(2)


       budconc = 0.0
       do n=1,nt
          nglob=n+offsetn !WP! offset is zero on levels domain
          do l=1,lmr
             lglob=l+offsetl !WP! offset is zero on tracer domain
             nzone_v=nzon_v(lglob)
             do j=1,jm(region)
                do i=1,im(region)
                   nzone=nzon(i,j)
                   budconc(nzone,nzone_v,nglob) = &
                        budconc(nzone,nzone_v,nglob) + &
                        rm(i,j,l,n)/ra(nglob)*1000.
                end do !i
             end do!j
          end do!l
       end do !n   
       !
       do  n=1,nt
          nglob=n+offsetn  !WP! offset is zero on levels domain
          do i=1,nbud
             do iy=1,nbud_v
                budini(i,iy,nglob)=budchng(i,iy,nglob)
                budchng(i,iy,nglob)=budconc(i,iy,nglob)-budini(i,iy,nglob)
             end do !iy
          end do !i
       end do !n

       nullify(rm)

#ifdef MPI   
       call gather_budget

       if ( myid == root_t ) then
#endif
          !
          ! write to hdf file
          !
          io = sfstart(trim(outdir)//'/budget_'//region_name(region)//'.hdf',DFACC_CREATE)
          istat = sfsnatt(io,'itau',     DFNT_INT64, 1, itau)
          istat = sfsnatt(io,'nbud',     DFNT_INT32, 1, nbud)
          istat = sfsnatt(io,'nbud_v',   DFNT_INT32, 1, nbud_v)
          istat = sfsnatt(io,'nreac',    DFNT_INT32, 1, nreac)
          istat = sfsnatt(io,'njnum',    DFNT_INT32, 1, njnum)
          istat = sfsnatt(io,'ntrace',   DFNT_INT32, 1, ntrace)
          istat = sfsnatt(io,'nmark',    DFNT_INT32, 1, nmark)
          istat = sfsnatt(io,'nhet',   DFNT_INT32, 1, nhet)
          istat = sfsnatt(io,'idate' ,   DFNT_INT32, 6, idate)
          istat = sfsnatt(io,'idatei' ,  DFNT_INT32, 6, idatei)
          istat = sfsnatt(io,'idatee' ,  DFNT_INT32, 6, idatee)
          istat = sfsnatt(io,'proces_nr' ,   DFNT_INT32, nproces, proces_nr)
          istat = sfscatt(io,'proces_name',DFNT_CHAR, nproces*10,proces_name)
          istat = sfscatt(io,'zone_name' ,DFNT_CHAR, nbud*20, zone_name)
          istat = sfscatt(io,'zone_namv' ,DFNT_CHAR, nbud_v*20, zone_namv)
          istat = sfscatt(io,'names'     ,DFNT_CHAR, ntrace*8, names)
          istat = sfscatt(io,'ratnam'    ,DFNT_CHAR, nreac*8, ratnam)
          istat = sfscatt(io,'jnam'      ,DFNT_CHAR, njnum*8, jnam)
          istat = sfscatt(io,'marknam'   ,DFNT_CHAR, nmark*8, marknam)
          istat = sfscatt(io,'rhetnam'     ,DFNT_CHAR, nhet*8, rhetnam)

#ifdef MPI     
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               njnum,'njnum',budrj_all,'budrj')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               nreac,'nreac',budrr_all,'budrr')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               nhet,'nhet',budrw_all,'budrw')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               nmark,'nmark',budmark_all,'budmark')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',buddry_all,'buddry')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budwet_lsp_all,'budwet_lsp')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budwet_cp_all,'budwet_cp')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budwet_nat_all,'budwet_nat')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budemi_all,'budemi')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budstrat_all,'budstrat')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budconc_all,'budconc')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budchng_all,'budchng')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budini_all,'budini')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntracet,'ntracet',budadvx_all,'budadvx')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntracet,'ntracet',budadvy_all,'budadvy')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntracet,'ntracet',budadvz_all,'budadvz')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntracet,'ntracet',budconv_all,'budconv')
          call io_write(io,im(nregions),'im',jm(nregions),'jm', &
               nbud_v,'nbud_v',ntrace,'ntrace', depwet_lsp_all, 'depwet_lsp')
          call io_write(io,im(nregions),'im',jm(nregions),'jm', &
               nbud_v,'nbud_v',ntrace,'ntrace', depwet_cp_all, 'depwet_cp')
          call io_write(io,im(nregions),'im',jm(nregions),'jm', &
               nbud_v,'nbud_v',ntrace,'ntrace', depwet_nat_all, 'depwet_nat')
          call io_write(io,im(nregions),'im',jm(nregions),'jm', &
               ntrace,'ntrace', depdry_all, 'depdry')
#else
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               njnum,'njnum',budrj,'budrj')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               nreac,'nreac',budrr,'budrr')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               nhet,'nhet',budrw,'budrw')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               nmark,'nmark',budmark,'budmark')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',buddry,'buddry')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budwet_lsp,'budwet_lsp')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budwet_cp,'budwet_cp')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budwet_nat,'budwet_nat')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budemi,'budemi')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budstrat,'budstrat')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budconc,'budconc')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budchng,'budchng')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntrace,'ntrace',budini,'budini')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntracet,'ntracet',budadvx,'budadvx')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntracet,'ntracet',budadvy,'budadvy')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntracet,'ntracet',budadvz,'budadvz')
          call io_write(io,nbud,'nbud',nbud_v,'nbud_v', &
               ntracet,'ntracet',budconv,'budconv')
          call io_write(io,im(nregions),'im',jm(nregions),'jm', &
               nbud_v,'nbud_v',ntrace,'ntrace', depwet_lsp, 'depwet_lsp')
          call io_write(io,im(nregions),'im',jm(nregions),'jm', &
               nbud_v,'nbud_v',ntrace,'ntrace', depwet_cp, 'depwet_cp')
          call io_write(io,im(nregions),'im',jm(nregions),'jm', &
               nbud_v,'nbud_v',ntrace,'ntrace', depwet_nat, 'depwet_nat')
          call io_write(io,im(nregions),'im',jm(nregions),'jm', &
               ntrace,'ntrace', depdry, 'depdry')
#endif
          print *, 'diagbud: Close budget file', sfend(io)

#ifdef MPI       
       end if
#endif
    case(3)
       !
       ! add wet and dry deposition fields to budget
       !
       do n=1,nt
          nglob=n+offsetn !WP! offset is zero on levels domain
          do nzone_v=1,nbud_v
             do j=1,jm(region)
                do i=1,im(region)
                   nzone=nzon(i,j)
                   budwet_lsp(nzone,nzone_v,nglob) = &
                        budwet_lsp(nzone,nzone_v,nglob) + &
                        depwet_lsp(i,j,nzone_v,nglob)
                   budwet_cp(nzone,nzone_v,nglob) = &
                        budwet_cp(nzone,nzone_v,nglob) + &
                        depwet_cp(i,j,nzone_v,nglob)
                   budwet_nat(nzone,nzone_v,nglob) = &
                        budwet_nat(nzone,nzone_v,nglob) + &
                        depwet_nat(i,j,nzone_v,nglob)
                end do !i
             end do!j
          end do!l
       end do !n
       ! dry deposition is always done paralel over layers (in chemistry)
       ! so the budget (depdry) is only present at one PE.
       ! This now is done on all PEs (many zeros) and later stuff is gathered.

       do n=1,ntrace
          do j=1,jm(region)
             do i=1,im(region)
                nzone=nzon(i,j)
                buddry(nzone,1,n)=buddry(nzone,1,n)+depdry(i,j,n)
             end do !i
          end do!j
       end do !n

    case default
       call escape_tm('diagbud: Wrong k entered the budget routines ' )
    end select


#ifdef MPI
  contains


    subroutine gather_budget
      implicit none
      integer :: nsend


      nsend=nbud*nbud_v*nmark
      call mpi_allreduce(budmark,budmark_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      nsend=nbud*nbud_v*nhet
      call mpi_allreduce(budrw,budrw_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      nsend=nbud*nbud_v*nreac
      call mpi_allreduce(budrr,budrr_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      nsend=nbud*nbud_v*njnum
      call mpi_allreduce(budrj,budrj_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      nsend=nbud*nbud_v*ntrace
      call mpi_allreduce(budini,budini_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(buddry,buddry_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budwet_lsp,budwet_lsp_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budwet_cp,budwet_cp_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budwet_nat,budwet_nat_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budemi,budemi_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budstrat,budstrat_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budconc,budconc_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budchng,budchng_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      nsend=nbud*nbud_v*ntracet
      call mpi_allreduce(budadvx,budadvx_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budadvy,budadvy_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budadvz,budadvz_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budconv,budconv_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      nsend = im(nregions)*jm(nregions)*nbud_v*ntrace
      call mpi_allreduce(depwet_cp,depwet_cp_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(depwet_nat,depwet_nat_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(depwet_lsp,depwet_lsp_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      nsend = im(nregions)*jm(nregions)*ntrace
      call mpi_allreduce(depdry,depdry_all,nsend,my_real, &
           mpi_sum,communicator,ierr)

    end subroutine gather_budget

#endif

  end subroutine diagbud



  subroutine budget_transport(region,stat,proces)
    !
    !
    !
    use dims
    use global_data, only : mass_dat
    use toolbox,     only : escape_tm
#ifdef MPI
    use mpi_const
#endif

    implicit none

    ! in/out
    integer,intent(in)   :: stat
    integer,intent(in)   :: region
    character(len=*)     :: proces

    ! local
    real,dimension(:,:,:,:),pointer  :: rm
    integer :: i,j,l,n,nzone,nzone_v,lmr,nt,offsetn,nglob

    ! start

#ifdef MPI
    if ( previous_par(region) /= 'tracer') &
         call escape_tm('budget_transport: must be on tracer domain')
#endif

    if ( region /= nregions ) return   !only for finest region

    rm => mass_dat(region)%rm_t 
    lmr=lm(region)
#ifdef MPI
    nt=ntracetloc
#else
    nt=ntracet
#endif

#ifdef MPI
    which_par='tracer'
    offsetn=sum(ntracet_ar(0:myid-1))
#else
    offsetn=0
#endif

    if ( stat == 0 ) then ! save current tracer concentrations
       rmold = 0.0
       do n=1,nt
          nglob=n+offsetn  
          do l=1,lmr
             nzone_v = nzon_v(l)
             do j=1,jm(region)
                do i=1,im(region)
                   nzone=nzon(i,j)
                   rmold(nzone,nzone_v,nglob) = &
                        rmold(nzone,nzone_v,nglob) + rm(i,j,l,n)
                end do
             end do
          end do
       end do
    else
       rmnew = 0.0
       do n=1,nt  !WP! local value of ntracet
          nglob=n+offsetn  
          do l=1,lmr
             nzone_v = nzon_v(l)
             do j=1,jm(region)
                do i=1,im(region)
                   nzone=nzon(i,j)
                   rmnew(nzone,nzone_v,nglob) = &
                        rmnew(nzone,nzone_v,nglob) + rm(i,j,l,n)
                end do
             end do
          end do
       end do

       select case ( proces )
       case('advx')
          do n=1,nt  !WP! local value of ntracet
             nglob=n+offsetn  
             budadvx(:,:,nglob) = budadvx(:,:,nglob) + &
                  (rmnew(:,:,nglob)-rmold(:,:,nglob))/ra(nglob)*1e3
          end do
       case('advy')
          do n=1,nt  !WP! local value of ntracet
             nglob=n+offsetn  
             budadvy(:,:,nglob) = budadvy(:,:,nglob) + &
                  (rmnew(:,:,nglob)-rmold(:,:,nglob))/ra(nglob)*1e3
          end do
       case('advz')
          do n=1,nt  !WP! local value of ntracet
             nglob=n+offsetn  
             budadvz(:,:,nglob) = budadvz(:,:,nglob) + &
                  (rmnew(:,:,nglob)-rmold(:,:,nglob))/ra(nglob)*1e3
          end do
       case('conv')
          do n=1,nt  !WP! local value of ntracet
             nglob=n+offsetn  
             budconv(:,:,nglob) = budconv(:,:,nglob) + &
                  (rmnew(:,:,nglob)-rmold(:,:,nglob))/ra(nglob)*1e3
          end do
       case default
          print*,'budget_transport: no valid value for transport budget', &
               ', budget not updated!'
       end select
    end if

    nullify(rm)

  end subroutine budget_transport


end module budget_fine
