!
! this module declares and deallocates the actual data
! dimensions are defined in module dims
! MK december 2002
! 
module global_data

  use dims        , only : region_data, wind_data, conv_data
  use dims        , only : nregions, im, jm, lm, nlon360, nlat180
  use dims        , only : okdebug, lmax_conv
  use chem_param  , only : ntracet
  use global_types, only : emis_data
  use global_types, only : d2_data, d23_data, d3_data
  use tracer_data , only : mass_data, mass_dat

#include "advect.h"

  implicit none
  
  ! --- in/out -----------------------------------

  private

  public  ::  use_convec

  ! variables
  
  ! NOTE: ifort compiler requires that both type and variable are public ..

  public  ::  rcfile, rcfile_runtime
  public  ::  inputdir
  public  ::  outdir
  
  public  ::  region_data, region_dat
  public  ::  wind_data, wind_dat
  public  ::  mass_data, mass_dat
  public  ::  conv_data, conv_dat
  public  ::  emis_data
  public  ::  d2_data, d23_data, d3_data

  ! subroutines
  public  ::  declare_fields, free_fields


  ! --- const ----------------------------------
  
  ! module name
  character(len=*), parameter  ::  mname = 'global_data'
  

  ! --- var ---------------------------------------------

  ! defintion of parameter 'rcfile' with rcfile name:
  include "rcfilename.h"

  ! path to input files:  
  character(len=256)  ::  inputdir = './'
  
  ! path to output files:
  character(len=256)  ::  outdir   = './'

  type(region_data),dimension(nregions),target   :: region_dat
  type(wind_data)  ,dimension(nregions),target   :: wind_dat

  logical       ::  use_convec
  type(conv_data)  ,dimension(nregions),target   :: conv_dat


contains


  ! ==========================================================

  
  !
  !    subroutine to allocate the memory for all the regions
  !    should be called at start of program
  !

  subroutine declare_fields

    use tracer_data, only : assign_massdat
    use partools, only : lmloc, allocate_mass

    ! local
    integer   :: region
    integer   :: imr,jmr,lmr

    ! start
    do region=1,nregions
       imr = im(region)
       jmr = jm(region)
       lmr = lm(region)
       allocate ( region_dat(region)%zoomed(imr,jmr)) 
       allocate ( region_dat(region)%edge(imr,jmr)) 
       allocate ( region_dat(region)%dxyp(jmr))
       
       ! fluxes through boundaries: 
       allocate ( wind_dat(region)%am_t(0:imr+1,0:jmr+1,0:lmr+1) )
       allocate ( wind_dat(region)%bm_t(0:imr+1,0:jmr+1,0:lmr+1) )
       allocate ( wind_dat(region)%cm_t(0:imr+1,0:jmr+1,0:lmr+1) )
       ! fill with zero for safety ...
       wind_dat(region)%am_t = 0.0
       wind_dat(region)%bm_t = 0.0
       wind_dat(region)%cm_t = 0.0
       
       allocate ( mass_dat(region)%phlb_t(imr,jmr,lmr+1    ) ) 
#ifdef MPI
       allocate ( mass_dat(region)%phlb_k(imr,jmr,lmloc+1    ) ) 
#endif
       if ( use_convec ) then
         allocate ( conv_dat(region)%dkg(imr,jmr,lmax_conv))
         allocate ( conv_dat(region)%cloud_base(imr,jmr))
         allocate ( conv_dat(region)%cloud_top (imr,jmr))
         allocate ( conv_dat(region)%cloud_lfs (imr,jmr))
         allocate ( conv_dat(region)%blh       (imr,jmr))
       end if

       ! WP! initialize mass_dat on tracer domain
       call assign_massdat( region, 1 )
       ! WP! initialize mass_dat on level domain
       if ( .not. allocate_mass ) call assign_massdat( region, 0 )

       if ( okdebug ) print *, 'declare_fields: Fields for grid, mass, ', &
            'and winds allocated, region:', region
    end do

  end subroutine declare_fields



  subroutine free_fields
    !
    !    subroutine to free the memory for all the regions
    !    should be called at end of program
    !
    use tracer_data, only : free_massdat
    use partools   , only : myid, allocate_mass

    implicit none

    ! local
    integer   :: region

    ! start
    do region=1,nregions
#ifdef MPI
       if ( okdebug ) print *, 'free_fields: deallocate... ', &
            myid, region,im(region),jm(region)       
#else
       if ( okdebug ) print *, 'free_fields: deallocate...', &
            region,im(region),jm(region)       
#endif

       deallocate ( region_dat(region)%zoomed )
       deallocate ( region_dat(region)%edge )
       deallocate ( region_dat(region)%dxyp )
#ifdef MPI
       deallocate ( mass_dat(region)%phlb_k )
#endif
       deallocate ( mass_dat(region)%phlb_t )
       deallocate ( wind_dat(region)%am_t )
       deallocate ( wind_dat(region)%bm_t )
       deallocate ( wind_dat(region)%cm_t )
       if ( use_convec ) then
         deallocate ( conv_dat(region)%dkg )
         deallocate ( conv_dat(region)%cloud_base )
         deallocate ( conv_dat(region)%cloud_top  )
         deallocate ( conv_dat(region)%cloud_lfs  )
         deallocate ( conv_dat(region)%blh        )
       end if
       ! WP! free mass_dat on tracer domain
       call free_massdat( region, 0 ) 
       ! WP! free mass_dat on level domain
       if ( .not. allocate_mass ) call free_massdat( region, 1 )

       if ( okdebug ) print *, 'free_fields: Fields for grid, mass, ', &
            'and winds deallocated, region:', myid, region
    end do

  end subroutine free_fields




end module global_data
